#include "SimHost.h"
#include <osg/Notify>

#include <OpenThreads/Thread>
#include <OpenThreads/Barrier>

// System includes
#include <cassert>

#ifdef _WIN32
#include <process.h>
#define getpid() _getpid()
#else
#include <unistd.h>
#endif 

// App includes


SimHost::SimHost()
{
	OSG_ALWAYS<<"SimHost instantiated."<<std::endl;
}

SimHost::~SimHost()
{
	_channelWorker.clear();
}

void SimHost::initialize()
{
	int numThreads = 6+1;	// 6 channels and one main thread
	OpenThreads::Barrier syncBarrier;

	// Create ChannelWorkers
	OSG_ALWAYS<<"Creating ChannelWorkers..."<<std::endl;
	for(int i=0; i<numThreads-1; ++i)
	{
		osg::ref_ptr<ChannelWorker> worker = new ChannelWorker(numThreads, syncBarrier);
		_channelWorker.push_back(worker);
	}
	OSG_ALWAYS<<"...done."<<std::endl;

	// Setup Threading
	OSG_ALWAYS << "Root Thread ID: " << getpid() << std::endl;

	OSG_ALWAYS<<OpenThreads::GetNumberOfProcessors()<<" CPUs found, using "<<(numThreads)<<" of them concurrently."<<std::endl;
	OpenThreads::Thread::SetConcurrency( numThreads );	// Only supported by pthread implementation, otherwise ignored

	OpenThreads::Thread::Init();
	
}

void SimHost::run()
{
	// Launch threads
    for(int i=0;i<static_cast<int>(_channelWorker.size());++i)
	{
		int status;
			
		//thread->addObserver(&observer);
		//thread->setStackSize(1024*256);
		status = _channelWorker[i]->start();
		assert(status == 0);
	}

	// Do something in the main thread
	for(int i=0;i<10000000;i++)
		OSG_ALWAYS<<"";
	
	// Signal threads to finish
    for(int i=0;i<static_cast<int>(_channelWorker.size());++i)
	{
		_channelWorker[i]->setDone(true);
		//_channelWorker[i]->cancel();		// Seems to be wrong
	}

	//syncBarrier.block(numThreads);  // Block until all threads are ready
}